module memory_ctrl (

	clk,
	sreset,	
	start_img,
	data_in,
	req,
	mem_status,
	avg_done,
	data_out,
	ram_data_load,
	ram_data_store,
	address,
	read,
	write,
	ce
	
);

input					clk; 
input					sreset;
input					start_img;
input[17:0]			data_in;
input 				req;
output reg[4:0]	mem_status;
output reg 			avg_done;
output reg[17:0]	data_out;

input[23:0]			ram_data_load;
output reg[23:0]	ram_data_store;
output[31:0]		address;
output reg 			write, read, ce;

reg[16:0] address_counter;
reg[3:0]	iteration_counter;

assign address = address_counter;


	// Declare the state register to be "safe" to implement
	// a safe state machine that can recover gracefully from
	// an illegal state (by returning to the reset state).
	(* syn_encoding = "safe" *) reg [2:0] state;
	
	// Declare states
	parameter LOADING = 0, STORING = 1, IDLE = 2, TRANSMITTING = 3, RESET = 4;
	
	parameter NO_OF_PIXELS = 81920;	// Number of pixels to store

	// IDLE = waiting to begin
	// STORING = storing data to RAM
	// LOADING = loading data from RAM
	// TRANSMITTING = sending data to I2C module as needed
	
	// Status signals
	always @ (state or address_counter) begin
		case (state)
			RESET:
			begin
				avg_done = 0;
				mem_status = 0;
			end
			
			IDLE:
			begin
				avg_done = 0;
				mem_status = 0;
			end
			
			LOADING:
			begin
				avg_done = 0;
				mem_status = 0;
			end
				
			STORING:
			begin
				avg_done = 0;
				mem_status = 0;
			end
			
			TRANSMITTING:
			begin
				if(address_counter < 31)
					mem_status = address_counter[4:0];
				else
					mem_status = 5'b11111;

			
				avg_done = 1;
			end
				
			default:
			begin
				avg_done = 0;
				mem_status = 0;
			end
		endcase
	end
	
	
	always @ (posedge clk) ram_data_store <= data_in; // result data from accumulator directly connected to RAM
	
	// data load control
	always @ (posedge clk)
	begin
		case(state)
			IDLE:
				data_out = data_out;

			
			STORING:
				data_out = data_out;
	
			LOADING:
				data_out = ram_data_load;

			TRANSMITTING:
				data_out = ram_data_load[16:4];	
		
		default:
				data_out = data_out;
		endcase
	end
	
	// memory control
	always @ (posedge clk) begin
		case (state)
			RESET:
			begin
				address_counter = 0;
				read = 0;
				write = 0;
				ce = 0;
			end
			
			IDLE:
			begin
				address_counter = 0;
				read = 0;
				write = 0;
			end
				
			LOADING:
			begin
				address_counter = address_counter;
				read = 1;
				write = 0;
				ce = 1;
			end
				
			STORING:
			begin
				address_counter = (address_counter + 1) % NO_OF_PIXELS;
				read = 0;
				write = 1;
				ce = 1;
			end
			
			TRANSMITTING: 
			begin
				ce = 0;
				
				if(req)
				begin
					address_counter = address_counter - 1;
					read = 1;
				end
				
				else
				begin
					address_counter = address_counter;
					read = 0;
				end
				
				read = 0;
			end
		
			default:
			begin
				address_counter = 0;
				read = 0;
				write = 0;
				ce = 0;
			end
		endcase
	end
	
	
	
	
	
	// iteration_counter control 
	always @ (posedge clk) begin
		case(state)
			RESET:
				iteration_counter = 0;
				
			IDLE:
				iteration_counter = 0;
				
			LOADING:
				iteration_counter = iteration_counter;
				
			STORING:
				if(address_counter == NO_OF_PIXELS - 1)
					iteration_counter = iteration_counter + 1;
				else
					iteration_counter = iteration_counter;
					
			TRANSMITTING: 
				iteration_counter = 0;
		
			default:
				iteration_counter = 0;
				
		endcase
	end
	
	
	// Determine the next state
	always @ (posedge clk) begin
		if (!sreset)
			state <= RESET;
		else
			case (state)
				RESET:
					state <= IDLE;
					
				IDLE:
					if(start_img)
						state <= LOADING;
					
				LOADING:
					state <= STORING;
					
				STORING:
					if((iteration_counter == 15) && (address_counter == NO_OF_PIXELS - 1))
						state <= TRANSMITTING;	
					else
						state <= LOADING;
					
				TRANSMITTING:
					if (address_counter == 0)
						state <= IDLE;	
					
				default:
					state <= RESET;
			endcase
	end

endmodule